#ifndef _IP_NAT_BIND_H
#define _IP_NAT_BIND_H
#include "ip_nat.h"

#define IP_NAT_BASE_CTL  	80	/* base for NAT socket options */
#define IP_NAT_SO_INSERT	IP_NAT_BASE_CTL
#define IP_NAT_SO_DELETE	(IP_NAT_BASE_CTL+1)
#define IP_NAT_SO_MAX		IP_NAT_SO_DELETE

/* These used by various modules. */
#define IP_NAT_UDP_ADD_USER	(IP_NAT_SO_MAX + 1)
#define IP_NAT_UDP_DELETE_USER	(IP_NAT_SO_MAX + 2)
#define IP_NAT_TCP_ADD_USER	(IP_NAT_SO_MAX + 3)
#define IP_NAT_TCP_DELETE_USER	(IP_NAT_SO_MAX + 4)

#define IP_NAT_BIND_MAX_NAMELEN 16

struct ip_nat_bind_user
{
	enum ip_nat_direction direction;

	struct ip_nat_tuple_proto tupleproto;
	/* Set ifname to "" for INCOMING from local process. */
	char ifname[IFNAMSIZ];

	/* Marks which bits we care about. */
	struct ip_nat_pktinfo mask;

	/* Hangs off end.  Double NUL char to terminate. */
	char typenames[0];
};

#ifdef __KERNEL__
struct iphdr;

struct ip_nat_perconn;

#define FIT_TO_TYPE(x,t) (((x)+(sizeof(t))-1)/(sizeof(t)))

struct ip_nat_bind
{
	struct ip_nat_bind *next;
	struct ip_nat_pktinfo info;
	struct ip_nat_pktinfo mask;

	u_int32_t precedence;

	enum ip_nat_direction direction;

	/* We keep this because pointer in ip_nat_pktinfo gets NULLed on
	 * device down. */
	char ifname[IFNAMSIZ];

	/* NULL-terminated array of bind types hang off the end. */
	struct ip_nat_bind_type *type[0];
};

struct ip_nat_bind_type
{
	struct ip_nat_bind_type *next;

	/* Name of this mapping. */
	const char name[IP_NAT_BIND_MAX_NAMELEN];

	/* Must be zero to unregister. */
	unsigned int usage;

	/* If you want your module count modified as rules use the type,
	   set this. */
	struct module *me;

	/* Will it accept this bind? */
	int (*check)(const struct ip_nat_bind *,
		     const struct ip_nat_bind_type **me);

	/* Function to call if we get a match: if NF_ACCEPT and perconn
	 * filled in, that perconn will be called. */
	unsigned int (*bind)(const struct iphdr *iph,
			     const struct ip_nat_pktinfo *pkt,
			     enum ip_nat_direction direction,
			     const struct device *dev,
			     struct ip_nat_address_range *range,
			     const struct ip_nat_bind *bind,
			     unsigned int typenumber,
			     struct ip_nat_perconn **perconn,
			     size_t extrasize);
};

extern int ip_nat_register_bind_type(struct ip_nat_bind_type *me);
extern int ip_nat_unregister_bind_type(struct ip_nat_bind_type *me);

#endif /*__KERNEL__*/
#endif /*_IP_NAT_BIND_H*/
