/* Header for use in defining a given protocol. */
#ifndef _IP_NAT_PROTO_H
#define _IP_NAT_PROTO_H

#include "ip_nat_bind.h"

struct ip_nat_proto
{
	/* Next pointer. */
	struct ip_nat_proto *next;

	/* Protocol number. */
	unsigned int proto;

	/* Protocol name */
	const char *name;

	/* Try to fill in the third arg; return true if possible. */
	int (*pkt_to_tuple)(const void *datah, 
			    size_t datalen, 
			    struct ip_nat_tuple *tuple);
	/* Do a packet translation according to the ip_nat_tuple_manip
	 * and direction (OUTGOING => src, INCOMING => dst). */
	void (*manip_pkt)(struct iphdr *hdr, 
			  size_t len,
			  const struct ip_nat_tuple_manip *manip,
			  enum ip_nat_direction direction);

	/* Is the per-proto part of the tuple in the per-proto part of
	 * this range? */
	int (*in_range)(const struct ip_nat_tuple *tuple, 
			const struct ip_nat_range *range);

	/* Allocate a unique IP_NAT_OUTGOING tuple if possible; return
           false if not. */
	int (*unique_tuple)(struct ip_nat_tuple_proto *tuplepr, 
			    const struct ip_nat_range *range);

	/* Invert the per-proto part of the tuple: turn xmit into reply. */
	void (*invert_tuple)(struct ip_nat_tuple *tuple);

	/* Print out the per-protocol part of the tuple. */
	unsigned int (*print_tuple)(char *buffer, const struct ip_nat_tuple *);

	/* Print out the per-protocol part of the manip. */
	unsigned int (*print_manip)(char *buffer, 
				    const struct ip_nat_tuple_manip *);

	/* Standard bind functions for this protocol. */
	struct ip_nat_bind_type bindtype;
};

/* Protocol registration. */
extern int ip_nat_register_proto(struct ip_nat_proto *proto);
extern int ip_nat_unregister_proto(struct ip_nat_proto *proto);

#endif /*_IP_NAT_PROTO_H*/
