/* The "unknown" protocol.  This is what is used for protocols we
 * don't understand.  It's returned by find_proto().
 */

/* FIXME: plug in unknowns if a proto field is NULL upon registration. */

static int unknown_pkt_to_tuple(const void *datah, size_t datalen, 
			     struct ip_nat_tuple *tuple)
{
	return 1;
}

static void unknown_tuple_to_pkt(struct iphdr *hdr, 
			      size_t len,
			      const struct ip_nat_tuple_manip *tuple,
			      enum ip_nat_direction direction)
{
	return;
}

static int unknown_in_range(const struct ip_nat_tuple *tuple, 
			 const struct ip_nat_range *range)
{
	return 1;
}

static int unknown_unique_tuple(struct ip_nat_tuple_proto *tuplepr, 
			     const struct ip_nat_range *range)
{
	struct ip_nat_perconn *perconn;

	/* If there's any other with this IP pair/proto, we can't
           allocate. */
	for (perconn = *hash_by_ips_proto(tuplepr->tuple.src.ip, 
					  tuplepr->tuple.dst.ip, 
					  tuplepr->proto);
	     perconn; 
	     perconn = perconn->by_ips_proto) {
		struct ip_nat_tuple_proto *tpr
			= &perconn->dir[IP_NAT_OUTGOING].connection;

		if (tpr->tuple.src.ip == tuplepr->tuple.src.ip
		    && tpr->tuple.dst.ip == tuplepr->tuple.dst.ip
		    && tpr->proto == tuplepr->proto) {
			DEBUGP("null: %u.%u.%u.%u->%u.%u.%u.%u %u taken\n",
			       IP_PARTS(tpr->tuple.src.ip),
			       IP_PARTS(tpr->tuple.dst.ip),
			       tpr->proto);
			return 0;
		}
	}
	return 1;
}

static void unknown_invert_tuple(struct ip_nat_tuple *tuple)
{
}

static unsigned int 
unknown_print_tuple(char *buffer, const struct ip_nat_tuple *tuple)
{
	return 0;
}

static unsigned int 
unknown_print_manip(char *buffer, const struct ip_nat_tuple_manip *tuple)
{
	return 0;
}

static unsigned int 
unknown_print_private(char *buffer, const struct ip_nat_perconn *perconn)
{
	return 0;
}

static int 
unknown_check(const struct ip_nat_binding *bind, unsigned int typenum)
{
	/* Don't let them explicitly bind to unknown type: use the generic
	 * to ensure they realize it's not supported. */
	DEBUGP("Attempt to bind with unknown type.\n");
	return 0;
}

static unsigned int 
unknown_bind(const struct iphdr *iph,
	     const struct ip_nat_pktinfo *pkt,
	     const struct ip_nat_binding_element *cause,
	     struct ip_nat_multi_range *range,
	     unsigned int typenumber,
	     struct ip_nat_perconn **perconn)
{
	/* We know nothing about this protocol, so keep connections
	   stored for 60 minutes since last packet. */
	*perconn = ip_nat_alloc_perconn(pkt, cause, range, 
					&ip_nat_refresh, &ip_nat_refresh, 
					HZ*60*60, 
					&ip_nat_delete_perconn_timeout);
	return NF_ACCEPT;
}

static void unknown_destroy(const struct ip_nat_perconn *perconn)
{
	return;
}

static struct ip_nat_proto unknown_proto = {
	NULL, 0, "unknown",
	unknown_pkt_to_tuple, 
	unknown_tuple_to_pkt, 
	unknown_in_range, 
	unknown_unique_tuple,
	unknown_invert_tuple,
	unknown_print_tuple,
	unknown_print_manip,
	{ NULL, "unknown", 0, NULL, 0, 0, 
	  unknown_print_private,
	  unknown_check, 
	  unknown_bind, 
	  unknown_destroy 
	}
};

